import numpy as np
import pandas as pd
import requests
from eucalyptus.config import BASE_API_URL
from eucalyptus.config import get_api_headers
import pandera as pa
import re


def log_training_data_from_pandas_simpler(model_name, model_version, features, labels):
    if isinstance(features, pd.DataFrame) and (isinstance(labels, pd.Series) or isinstance(labels, pd.DataFrame)):
        training_data_distribution = []

        for column in features:
            column_name = column.lower()
            column_name = column_name.strip()
            column_name = re.sub(r'\W+', ' ', column_name)
            column_name = "_".join(column_name.split())
            num_unique_values = len(features[column].unique())
            if num_unique_values <= 10:
                value_counts = features[column].value_counts()
                training_bins = []
                training_hist = []

                for index, value in value_counts.items():
                    training_bins.append(index)
                    training_hist.append(value)

                histogram_info = {"training_hist": training_hist,
                                  "training_bins": training_bins}
                training_data_distribution.append({column_name: histogram_info, "feature_type": "categorical"})

            else:
                min_training = features[column].values.min()
                max_training = features[column].values.max()

                step = (max_training - min_training) / 10
                step = round(step, 2)

                training_hist, training_bins = np.histogram(features[column].values,
                                                            bins=np.arange(min_training,
                                                                           max_training + step,
                                                                           step))

                training_hist = np.around(training_hist, 2).tolist()
                training_bins = np.around(training_bins, 2).tolist()

                histogram_info = {"training_hist": training_hist,
                                  "training_bins": training_bins, "min_training": min_training,
                                  "max_training": max_training}

                training_data_distribution.append({column_name: histogram_info, "feature_type": "continuous"})

        payload = {"model_name": model_name,
                   "model_version": model_version,
                   "training_data_distribution": training_data_distribution,
                }

        print(payload)

        response = requests.post(f'{BASE_API_URL}/uploadTrainingDataFromSDK',
                                 json=payload, headers=get_api_headers())
        return response.json()
    else:
        raise ValueError("features and labels must be pandas dataframes")


def infer_schema(features, labels):
    features_schema = pa.infer_schema(features)
    if isinstance(features_schema, pa.schemas.DataFrameSchema):
        features_schema = features_schema.dtypes
    else:
        features_schema = features_schema.dtype
    f_schema = []
    for key, value in features_schema.items():
        column_name = key.lower()
        column_name = column_name.strip()
        column_name = re.sub(r'\W+', ' ', column_name)
        column_name = "_".join(column_name.split())

        f_schema.append({"col_name": column_name, "col_type": value.__str__()})

    l_schema = []
    labels_schema = pa.infer_schema(labels)
    if isinstance(labels_schema, pa.schemas.SeriesSchema):
        schema = labels_schema.dtype
        label_column_name = labels_schema.name
        label_column_name = label_column_name.lower()
        label_column_name = label_column_name.strip()
        label_column_name = re.sub(r'\W+', ' ', label_column_name)
        label_column_name = "_".join(label_column_name.split())
        l_schema.append({"col_name": label_column_name, "col_type": str(schema)})

    else:
        labels_schema = labels_schema.dtypes
        for key, value in labels_schema.items():
            label_column_name = key.lower()
            label_column_name = label_column_name.strip()
            label_column_name = re.sub(r'\W+', ' ', label_column_name)
            label_column_name = "_".join(label_column_name.split())

            l_schema.append({"col_name": label_column_name, "col_type": value.__str__()})

    return f_schema, l_schema


def validate_model_name(name):
    model_name = name.strip()
    pattern = '^[a-z][a-z_]+[a-z]$'
    model_name = re.sub(r"\__+", "_", model_name)
    result = re.match(pattern, model_name)
    if result:
        return True
    else:
        return False




from eucalyptus.utils.model_logger import log_model_prediction, log_model_actual, create_new_model_version
from eucalyptus.utils.utils import log_training_data_from_pandas_simpler
from eucalyptus.config import get_api_headers
from eucalyptus.config import set_api_token
from eucalyptus.config import BASE_API_URL
from eucalyptus.utils.utils import infer_schema, validate_model_name
import requests


class bcolors:
    OK = '\033[92m' #GREEN
    WARNING = '\033[93m' #YELLOW
    FAIL = '\033[91m' #RED
    RESET = '\033[0m' #RESET COLOR


def initialize(api_token: str):
    set_api_token(api_token)
    print("api token set")


def create_new_model(model_name, model_version, model_type, features, labels):
    if not validate_model_name(model_name):
        print(bcolors.FAIL + "Error: Invalid model name. The model name can only contain lowercase letters with underscores" + bcolors.RESET)
        return
    feature_schema, label_schema = infer_schema(features, labels)
    data = {"model_name": model_name, "model_version": str(model_version), "model_type": model_type,
            "feature_schema": feature_schema, "label_schema": label_schema}
    response = requests.post(f'{BASE_API_URL}/registerNewModelSDK', json=data, headers=get_api_headers())
    response = response.json()
    if response['status'] != 200:
        print(bcolors.FAIL + response['msg'] + bcolors.RESET)
    else:
        print(response['msg'])


def create_new_version(model_name, new_version_name):
    create_new_model_version(model_name, new_version_name)


def log_prediction(model_name, model_version, features, prediction):
    log_model_prediction(model_name=model_name, model_version=model_version, features=features, prediction=prediction)


def log_actual(model_name, request_ids, actual):
    log_model_actual(model_name=model_name, request_id=request_ids, actual=actual)


def log_training_data(model_name, model_version, features, labels):
    log_training_data_from_pandas_simpler(model_name, model_version, features, labels)



from eucalyptus.config import BASE_API_URL
from eucalyptus.config import get_api_headers
import json
import pandas as pd
import sys
import requests


def log_model_prediction(model_name, model_version, features, prediction):
    features = {k.lower(): v for k, v in features.items()}
    data = {"model_name": model_name, "model_version": str(model_version), "features": features, "prediction": prediction}
    response = requests.post(f'{BASE_API_URL}/logPrediction', json=data, headers=get_api_headers())


def log_model_actual(model_name, request_id, actual):
    if isinstance(request_id, pd.Series) and isinstance(actual, pd.Series):
        zipped_data = dict(zip(request_id, actual))
        # Check if the data is larger than 1 MB in size because that is the default limit
        if sys.getsizeof(zipped_data) >= pow(2, 20):
            raise Exception("The size of the data is too large, please send the data in smaller chunks")

        data = {"model_name": model_name, "data": zipped_data}
        response = requests.post(f'{BASE_API_URL}/logActual', json=data, headers=get_api_headers())

    elif isinstance(request_id, list) and isinstance(actual, list):
        zipped_data = dict(zip(request_id, actual))
        # Check if the data is larger than 1 MB in size because that is the default limit
        if sys.getsizeof(zipped_data) >= pow(2, 20):
            raise Exception("The size of the data is too large, please send the data in smaller chunks")

        data = {"model_name": model_name, "data": zipped_data}
        response = requests.post(f'{BASE_API_URL}/logActual', json=data, headers=get_api_headers())
    else:
        print("The arguments request_id and actual can ONLY accept a list or a pandas series object")


def create_new_model_version(model_name, new_version_name):
    data = {"model_name": model_name, "version": new_version_name}
    response = requests.post(f'{BASE_API_URL}/createNewModelVersion', json=data, headers=get_api_headers())




from setuptools import setup, find_packages


setup(
    name='eucalyptus',
    version='0.0.2',
    license='MIT',
    author="Het Trivedi",
    author_email='het.trivedi05@gmail.com',
    packages=find_packages(include=['eucalyptus', 'eucalyptus.*']),
    url='https://github.com/htrivedi99/eucalyptus-sdk',
    install_requires=[
          'requests', 'pandas', 'numpy', 'pandera'
      ],

)


from configparser import ConfigParser
from pathlib import Path
import os

CONFIG_FILE_PATH = ".eucalyptus_config"


def check_config_file_exists():
    try:
        config = ConfigParser()
        config.read(CONFIG_FILE_PATH)
        return os.path.exists(CONFIG_FILE_PATH)
    except Exception as e:
        return False


def set_default_config():
    exists = check_config_file_exists()
    if not exists:
        config = ConfigParser()
        if not config.has_section('api'):
            config.add_section('api')
        config.set('api', 'base_api_url', 'https://eucalyptus.ai/api')
        with open(CONFIG_FILE_PATH, 'w') as conf:
            config.write(conf)


def read_config():
    if not check_config_file_exists():
        set_default_config()
    config = ConfigParser()
    config.read(CONFIG_FILE_PATH)
    return config


def set_config_values(section, key, value):
    config = read_config()
    if section not in config.sections():
        config.add_section(section)
    config.set(section, key, value)
    with open(CONFIG_FILE_PATH, 'w') as conf:
        config.write(conf)


def set_api_token(api_token):
    try:
        set_config_values('api', 'api_token', api_token)
    except Exception as e:
        print(e)
        set_default_config()


def get_api_headers():
    config = read_config()
    try:
        api_token = config.get('api', 'api_token')
        headers = {'X-API-TOKEN': api_token}
        return headers
    except Exception as e:
        print(e)
        set_default_config()


def get_base_api_url():
    config = read_config()
    try:
        base_api_url = config.get('api', 'base_api_url')
        return base_api_url
    except Exception as e:
        print(e)
        set_default_config()


BASE_API_URL = get_base_api_url()



from .sdk_functions import initialize
from .sdk_functions import create_new_model
from .sdk_functions import create_new_version
from .sdk_functions import log_prediction
from .sdk_functions import log_actual
from .sdk_functions import log_training_data
